@@ -13,7 +13,9 @@ import logging
 import base64
 import itertools
 import chardet
-from pydocx import PyDocX
+import mammoth
+import pypandoc
+import bs4
 from bs4 import BeautifulSoup
 from parse import parse
 import html2text
@@ -29,7 +31,8 @@ from chgksuite.common import (
     DefaultNamespace,
     check_question,
     QUESTION_LABELS,
-    get_source_dirs
+    get_source_dirs,
+    compose_4s
 )
 from chgksuite.composer import gui_compose
 
@@ -510,83 +513,95 @@ def generate_imgname(target_dir, ext):
 
 def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
     target_dir = os.path.dirname(os.path.abspath(docxfile))
-    input_docx = (
-        PyDocX.to_html(docxfile)
-        .replace("</strong><strong>", "")
-        .replace("</em><em>", "")
-        .replace("_", "$$$UNDERSCORE$$$")
-    )
-    bsoup = BeautifulSoup(input_docx, "html.parser")
-
-    if debug:
-        with codecs.open(
-            os.path.join(target_dir, "debug.pydocx"), "w", "utf8"
-        ) as dbg:
-            dbg.write(input_docx)
-
-    for tag in bsoup.find_all("style"):
-        tag.extract()
-    for tag in bsoup.find_all("p"):
-        if tag.string:
-            tag.string = tag.string + SEP
-    for tag in bsoup.find_all("b"):
-        tag.unwrap()
-    for tag in bsoup.find_all("strong"):
-        tag.unwrap()
-    for tag in bsoup.find_all("i"):
-        tag.string = "_" + tag.get_text() + "_"
-        tag.unwrap()
-    for tag in bsoup.find_all("em"):
-        tag.string = "_" + tag.get_text() + "_"
-        tag.unwrap()
-    if args.fix_spans:
-        for tag in bsoup.find_all("span"):
-            tag.unwrap()
-    for h in ["h1", "h2", "h3", "h4"]:
-        for tag in bsoup.find_all(h):
-            tag.unwrap()
-    for tag in bsoup.find_all("li"):
-        if tag.string:
-            tag.string = "- " + tag.string
-    for tag in bsoup.find_all("img"):
-        imgparse = parse("data:image/{ext};base64,{b64}", tag["src"])
-        imgname = generate_imgname(target_dir, imgparse["ext"])
-        with open(os.path.join(target_dir, imgname), "wb") as f:
-            f.write(base64.b64decode(imgparse["b64"]))
-        imgpath = os.path.basename(imgname)
-        tag.insert_before("(img {})".format(imgpath))
-        tag.extract()
-    for tag in bsoup.find_all("hr"):
-        tag.extract()
-    if args.links == "unwrap":
-        for tag in bsoup.find_all("a"):
-            tag.unwrap()
-    elif args.links == "old":
-        for tag in bsoup.find_all("a"):
-            if not tag.string or rew(tag.string) == "":
-                tag.extract()
-            else:
-                tag.string = tag["href"]
-                tag.unwrap()
-
-    if debug:
-        with codecs.open(
-            os.path.join(target_dir, "debug_raw.html"), "w", "utf8"
-        ) as dbg:
-            dbg.write(str(bsoup))
-        with codecs.open(
-            os.path.join(target_dir, "debug.html"), "w", "utf8"
-        ) as dbg:
-            dbg.write(bsoup.prettify())
-
-    h = html2text.HTML2Text()
-    h.body_width = 0
-
-    if args.bs_prettify:
-        html2text_input = bsoup.prettify()
+    if args.parsing_engine == "pypandoc":
+        txt = pypandoc.convert_file(docxfile, "plain", extra_args=["--wrap=none"])
     else:
-        html2text_input = str(bsoup)
-    txt = h.handle(html2text_input)
+        with open(docxfile, "rb") as docx_file:
+            html = mammoth.convert_to_html(docx_file).value
+        input_docx = (
+            html
+            .replace("</strong><strong>", "")
+            .replace("</em><em>", "")
+            .replace("_", "$$$UNDERSCORE$$$")
+        )
+        bsoup = BeautifulSoup(input_docx, "html.parser")
+
+        if debug:
+            with codecs.open(
+                os.path.join(target_dir, "debug.pydocx"), "w", "utf8"
+            ) as dbg:
+                dbg.write(input_docx)
+
+        for tag in bsoup.find_all("style"):
+            tag.extract()
+        for tag in bsoup.find_all("p"):
+            if tag.string:
+                tag.string = tag.string + SEP
+        for tag in bsoup.find_all("b"):
+            tag.unwrap()
+        for tag in bsoup.find_all("strong"):
+            tag.unwrap()
+        for tag in bsoup.find_all("i"):
+            tag.string = "_" + tag.get_text() + "_"
+            tag.unwrap()
+        for tag in bsoup.find_all("em"):
+            tag.string = "_" + tag.get_text() + "_"
+            tag.unwrap()
+        if args.fix_spans:
+            for tag in bsoup.find_all("span"):
+                tag.unwrap()
+        for h in ["h1", "h2", "h3", "h4"]:
+            for tag in bsoup.find_all(h):
+                tag.unwrap()
+        for tag in bsoup.find_all("li"):
+            if tag.string:
+                tag.string = "- " + tag.string
+        for tag in bsoup.find_all("img"):
+            imgparse = parse("data:image/{ext};base64,{b64}", tag["src"])
+            imgname = generate_imgname(target_dir, imgparse["ext"])
+            with open(os.path.join(target_dir, imgname), "wb") as f:
+                f.write(base64.b64decode(imgparse["b64"]))
+            imgpath = os.path.basename(imgname)
+            tag.insert_before("(img {})".format(imgpath))
+            tag.extract()
+        for tag in bsoup.find_all("hr"):
+            tag.extract()
+        if args.links == "unwrap":
+            for tag in bsoup.find_all("a"):
+                tag.unwrap()
+        elif args.links == "old":
+            for tag in bsoup.find_all("a"):
+                if not tag.string or rew(tag.string) == "":
+                    tag.extract()
+                else:
+                    tag.string = tag["href"]
+                    tag.unwrap()
+
+        if debug:
+            with codecs.open(
+                os.path.join(target_dir, "debug_raw.html"), "w", "utf8"
+            ) as dbg:
+                dbg.write(str(bsoup))
+            with codecs.open(
+                os.path.join(target_dir, "debug.html"), "w", "utf8"
+            ) as dbg:
+                dbg.write(bsoup.prettify())
+
+        h = html2text.HTML2Text()
+        h.body_width = 0
+
+        if args.parsing_engine == "bs_prettify":
+            html2text_input = bsoup.prettify()
+            txt = h.handle(html2text_input)
+        elif args.parsing_engine == "hard_unwrap":
+            for tag in bsoup:
+                if isinstance(tag, bs4.element.Tag):
+                    tag.unwrap()
+            txt = bsoup.prettify()
+        else:
+            html2text_input = str(bsoup)
+            txt = h.handle(html2text_input)
+
     txt = (
         txt.replace("\\-", "")
         .replace("\\.", ".")
@@ -613,101 +628,6 @@ def chgk_parse_docx(docxfile, defaultauthor="", regexes=None, args=None):
     return final_structure
 
 
-def remove_double_separators(s):
-    return re.sub(r"({})+".format(SEP), SEP, s)
-
-
-def compose_4s(structure, args=None):
-    types_mapping = {
-        "meta": "# ",
-        "tour": "## ",
-        "tourrev": "## ",
-        "editor": "#EDITOR ",
-        "heading": "### ",
-        "ljheading": "###LJ ",
-        "date": "#DATE ",
-        "question": "? ",
-        "answer": "! ",
-        "zachet": "= ",
-        "nezachet": "!= ",
-        "source": "^ ",
-        "comment": "/ ",
-        "author": "@ ",
-        "handout": "> ",
-        "Question": None,
-    }
-
-    def format_element(z):
-        if isinstance(z, str):
-            return remove_double_separators(z)
-        elif isinstance(z, list):
-            if isinstance(z[1], list):
-                return (
-                    remove_double_separators(z[0])
-                    + SEP
-                    + "- "
-                    + ("{}- ".format(SEP)).join(
-                        ([remove_double_separators(x) for x in z[1]])
-                    )
-                )
-            else:
-                return (
-                    SEP
-                    + "- "
-                    + ("{}- ".format(SEP)).join(
-                        [remove_double_separators(x) for x in z]
-                    )
-                )
-
-    def tryint(s):
-        try:
-            return int(s)
-        except (TypeError, ValueError):
-            return
-
-    def is_zero(s):
-        return str(s).startswith("0") or not tryint(s)
-
-    result = ""
-    first_number = True
-    for element in structure:
-        if element[0] in types_mapping and types_mapping[element[0]]:
-            result += (
-                types_mapping[element[0]]
-                + format_element(element[1])
-                + SEP
-                + SEP
-            )
-        elif element[0] == "Question":
-            tmp = ""
-            overrides = element[1].get("overrides") or {}
-            if "number" in element[1]:
-                if args.numbers_handling == "default":
-                    if is_zero(element[1]["number"]):
-                        tmp += "№ " + element[1]["number"] + SEP
-                    elif first_number and tryint(element[1]["number"]) > 1:
-                        tmp += "№№ " + element[1]["number"] + SEP
-                elif args.numbers_handling == "all":
-                    tmp += "№ " + element[1]["number"] + SEP
-                if not is_zero(element[1]["number"]):
-                    first_number = False
-            for label in QUESTION_LABELS:
-                override_label = "" if label not in overrides else (
-                    "!!{} ".format(overrides[label])
-                )
-                if label in element[1] and label in types_mapping:
-                    tmp += (
-                        types_mapping[label]
-                        + override_label
-                        + format_element(element[1][label])
-                        + SEP
-                    )
-            tmp = re.sub(r"{}+".format(SEP), SEP, tmp)
-            tmp = tmp.replace("\r\r", "\r")
-            result += tmp + SEP
-    return result
-
-
 def chgk_parse_wrapper(path, args, regexes=None):
     abspath = os.path.abspath(path)
     target_dir = os.path.dirname(abspath)
